# How To Write An Application To Host on OpenShift

This guide provides information relevant to application developers who wish
to integrate their code with OpenShift and the cartridges which drive their
applications.

The [Cartridge Author Guide](README.writing_cartridges.md) also contains
helpful information for application developers who desire a more complete understanding
of cartridges and their interaction with application code.

## Environment Variables

OpenShift builds the execution environment which is made available to application
scripts and other code. The environment contains a variety of variables related to
OpenShift and the application itself which can only be determined at runtime. In
addition to the standard environment variables exposed to the application, cartridges
may also expose custom variables for the application to consume; consult the specific
cartridge documentation for more information about what the cartridge makes available.

### Standard OpenShift Environment Variables

The following is a comprehensive list of the predefined environment variables available
regardless of the cartridges in use for a given application.

  * `HOME`                alias for `OPENSHIFT_HOMEDIR`
  * `HISTFILE`            bash history file
  * `OPENSHIFT_APP_DNS`   the application's fully qualified domain name that your cartridge is a part of
  * `OPENSHIFT_APP_NAME`  the validated user assigned name for the application. Black list is system dependent.
  * `OPENSHIFT_APP_UUID`  OpenShift assigned UUID for the application
  * `OPENSHIFT_DATA_DIR`  the directory where your cartridge may store data
  * `OPENSHIFT_GEAR_DNS`  the gear's fully qualified domain name that your cartridge is a part of. May or may
                          not be equal to `OPENSHIFT_APP_DNS`
  * `OPENSHIFT_GEAR_NAME` OpenShift assigned name for the gear. May or may not be equal to `OPENSHIFT_APP_NAME`
  * `OPENSHIFT_GEAR_UUID` OpenShift assigned UUID for the gear
  * `OPENSHIFT_HOMEDIR`   OpenShift assigned directory for the gear
  * `OPENSHIFT_REPO_DIR`  the directory where the developer's application is "archived" to and will be run from.
  * `OPENSHIFT_TMP_DIR`   the directory where your cartridge may store temporary data
  * `TMPDIR`              alias for `OPENSHIFT_TMP_DIR`
  * `TMP`                 alias for `OPENSHIFT_TMP_DIR`

### Cartridge Specific Environment Variables

Although cartridges may expose further arbitrarily named environment variables, there are
several variables which are generated by OpenShift for every cartridge based on well-defined
standards incorporating cartridge metadata.

To learn more about these variables, consult the following pieces of documentation from the
Cartridge Author guide.

  * [Exposing Services / TCP Endpoints](README.writing_cartridges.md#exposing-services--tcp-endpoints)
  * [Environment Variables](README.writing_cartridges.md#environment-variables)

## Application Configuration

OpenShift applications are tightly integrated with Git. The application's Git
repository contains not only the code and configuration for the application, but
also special files which help integrate the application OpenShift itself.

### Action Hooks

OpenShift provides application developers entry points into various application
and platform lifecycle operations. These entry points are referred to as
"action hooks", and have a special location within the application's Git
repository:

`<repository>/.openshift/action_hooks`

During any OpenShift process which supports an action hook, the application 
action hook directory will be checked for an executable file matching the
specified name. If such a file is present, it will be executed before returning
control to the process.

#### Cartridge Control Action Hooks

Cartridges implement a standard set of named control actions which allow them to
function within OpenShift. Each time OpenShift invokes one of these cartridge
actions, a standard set of application action hooks are executed to give the
application developer an opportunity to integrate more closely with specific
cartridges.

The following is a list of all possible action hooks executed in association
with a single cartridge control action. For each control action, a set of `pre`
and `post` action hooks surround the control action. In the action hook names,
`{Cartridge-Short-Name}` refers to the `Cartridge-Short-Name` manifest element
of the cartridge.

- `start` control action:
  - `pre_start`
  - `pre_start_{Cartridge-Short-Name}`
  - `post_start`
  - `post_start_{Cartridge-Short-Name}`
- `stop` control action: 
  - `pre_stop`
  - `pre_stop_{Cartridge-Short-Name}`
  - `post_stop`
  - `post_stop_{Cartridge-Short-Name}`
- `reload` control action:
  - `pre_reload`
  - `pre_reload_{Cartridge-Short-Name}`
  - `post_reload`
  - `post_reload_{Cartridge-Short-Name}`
- `restart` control action: 
  - `pre_restart`
  - `pre_restart_{Cartridge-Short-Name}`
  - `post_restart`
  - `post_restart_{Cartridge-Short-Name}`
- `tidy` control action:
  - `pre_tidy`
  - `pre_tidy_{Cartridge-Short-Name}`
  - `post_tidy`
  - `post_tidy_{Cartridge-Short-Name}`

For details about the control actions (including what they represent and when
they are called), refer to the [control script documentation](README.writing_cartridges.md#bincontrol)
in the [Writing Cartridges](README.writing_cartridges.md) guide.

#### Build Action Hooks

During a Git push, applications using the default OpenShift build lifecycle 
are given an opportunity to participate in the build/deploy workflow via another
set of action hooks. The workflow and sequence of actions for the build lifecycle
is described in detail in the [OpenShift Builds](README.writing_cartridges.md#openshift-builds)
section of the [Writing Cartridges](README.writing_cartridges.md) guide.

The list of action hooks supported during the default build lifecycle are:

- `pre-build`
- `build`
- `deploy`
- `post-deploy`

Refer to the [OpenShift Builds](README.writing_cartridges.md#openshift-builds)
guide for details about when these action hooks are invoked.

#### Custom Cartridge Action Hooks

Cartridges may support custom action hooks available to application developers.
Consult the documentation for individual cartridges to learn which hooks are
supported and how to correctly implement them.
